#  NanoVNASaver
#
#  A python program to view and export Touchstone data from a NanoVNA
#  Copyright (C) 2019, 2020  Rune B. Broberg
#  Copyright (C) 2020,2021 NanoVNA-Saver Authors
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
import logging
from time import sleep
from typing import List, Iterator, Set

from PyQt5 import QtGui

from marge.vna.Version import Version
from marge.vna.Serial import Interface, drain_serial

logger = logging.getLogger(__name__)

DISLORD_BW = {
    10: 363,
    33: 117,
    50: 78,
    100: 39,
    200: 19,
    250: 15,
    333: 11,
    500: 7,
    1000: 3,
    2000: 1,
    4000: 0,
}
WAIT = 0.05


def _max_retries(bandwidth: int, datapoints: int) -> int:
    return round(20 + 20 * (datapoints / 101) +
                 (1000 / bandwidth) ** 1.30 * (datapoints / 101))


class VNA:
    name = "VNA"
    valid_datapoints = (101, 51, 11)
    wait = 0.05

    def __init__(self, iface: Interface):
        self.serial = iface
        self.version = Version("0.0.0")
        self.features = set()
        self.validateInput = False
        self.datapoints = self.valid_datapoints[0]
        self.bandwidth = 1000
        self.bw_method = "ttrftech"
        self.sweep_max_freq_Hz = None
        # [((min_freq, max_freq), [description]]. Order by increasing
        # frequency. Put default output power first.
        self.txPowerRanges = []
        if self.connected():
            self.version = self.readVersion()
            self.read_features()
            logger.debug("Features: %s", self.features)
            #  cannot read current bandwidth, so set to highest
            #  to get initial sweep fast
            if "Bandwidth" in self.features:
                self.set_bandwidth(self.get_bandwidths()[-1])

    def connect(self):
        logger.info("connect %s", self.serial)
        with self.serial.lock:
            self.serial.open()

    def disconnect(self):
        logger.info("disconnect %s", self.serial)
        with self.serial.lock:
            self.serial.close()

    def reconnect(self):
        self.disconnect()
        sleep(WAIT)
        self.connect()
        sleep(WAIT)

    def exec_command(self, command: str, wait: float = WAIT) -> Iterator[str]:
        logger.debug("exec_command(%s)", command)
        with self.serial.lock:
            drain_serial(self.serial)
            self.serial.write(f"{command}\r".encode('ascii'))
            sleep(wait)
            retries = 0
            max_retries = _max_retries(self.bandwidth, self.datapoints)
            logger.debug("Max retries: %s", max_retries)
            while True:
                line = self.serial.readline()
                line = line.decode("ascii").strip()
                if not line:
                    retries += 1
                    if retries > max_retries:
                        raise IOError("too many retries")
                    sleep(wait)
                    continue
                if line == command:  # suppress echo
                    continue
                if line.startswith("ch>"):
                    logger.debug("Needed retries: %s", retries)
                    break
                yield line

    def read_features(self):
        result = " ".join(self.exec_command("help")).split()
        logger.debug("result:\n%s", result)
        if "capture" in result:
            self.features.add("Screenshots")
        if "bandwidth" in result:
            self.features.add("Bandwidth")
            result = " ".join(list(self.exec_command("bandwidth")))
            if "Hz)" in result:
                self.bw_method = "dislord"
        if len(self.valid_datapoints) > 1:
            self.features.add("Customizable data points")

    def get_bandwidths(self) -> List[int]:
        logger.debug("get bandwidths")
        if self.bw_method == "dislord":
            return list(DISLORD_BW.keys())
        result = " ".join(list(self.exec_command("bandwidth")))
        try:
            result = result.split(" {")[1].strip("}")
            return sorted([int(i) for i in result.split("|")])
        except IndexError:
            return [1000, ]

    def set_bandwidth(self, bandwidth: int):
        bw_val = DISLORD_BW[bandwidth] \
            if self.bw_method == "dislord" else bandwidth
        result = " ".join(self.exec_command(f"bandwidth {bw_val}"))
        if self.bw_method == "ttrftech" and result:
            raise IOError(f"set_bandwith({bandwidth}: {result}")
        self.bandwidth = bandwidth

    def readFrequencies(self) -> List[int]:
        return [int(f) for f in self.readValues("frequencies")]

    def resetSweep(self, start: int, stop: int):
        pass

    def _get_running_frequencies(self):
        """
        If possible, read frequencies already running
        if not return default values
        Overwrite in specific HW
        """
        return 27000000, 30000000

    def connected(self) -> bool:
        return self.serial.is_open

    def getFeatures(self) -> Set[str]:
        return self.features

    def getCalibration(self) -> str:
        return " ".join(list(self.exec_command("cal")))

    def getScreenshot(self) -> QtGui.QPixmap:
        return QtGui.QPixmap()

    def flushSerialBuffers(self):
        if not self.connected():
            return
        with self.serial.lock:
            self.serial.write("\r\n\r\n".encode("ascii"))
            sleep(0.1)
            self.serial.reset_input_buffer()
            self.serial.reset_output_buffer()
            sleep(0.1)

    def readFirmware(self) -> str:
        result = "".join(list(self.exec_command("info")))
        logger.debug("result:\n%s", result)
        return result

    def readValues(self, value) -> List[str]:
        logger.debug("VNA reading %s", value)
        result = list(self.exec_command(value))
        logger.debug("VNA done reading %s (%d values)",
                     value, len(result))
        return result

    def readVersion(self) -> 'Version':
        result = list(self.exec_command("version"))
        logger.debug("result:\n%s", result)
        return Version(result[0])

    def setSweep(self, start, stop):
        list(self.exec_command(f"sweep {start} {stop} {self.datapoints}"))

    def setTXPower(self, freq_range, power_desc):
        raise NotImplementedError()



"""
:author:    J.M. Algarín
:email:     josalggui@i3m.upv.es
:affiliation: MRILab, i3M, CSIC, Valencia, Spain

"""
import os
import platform
import subprocess
from datetime import datetime

import qdarkstyle
from PyQt5.QtGui import QPixmap

from marge.widgets.widget_toolbar_figures import FiguresToolBar


class FiguresController(FiguresToolBar):
    """
    Controller class for managing figures and screenshots.

    Args:
        *args: Variable length argument list.
        **kwargs: Arbitrary keyword arguments.

    Inherits:
        FiguresToolBar: Base class for the figures toolbar.
    """
    def __init__(self, *args, **kwargs):
        """
        Initializes the FiguresController.

        Args:
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.
        """
        super(FiguresController, self).__init__(*args, **kwargs)

        self.action_full_screen.setCheckable(True)
        self.action_full_screen.triggered.connect(self.doFullScreen)
        self.action_screenshot.triggered.connect(self.doScreenshot)
        self.action_open_directory.triggered.connect(self.open_folder)
        self.action_postprocessing.triggered.connect(self.openPostGui)
        self.switch_theme_action.triggered.connect(self.switch_theme)

    def switch_theme(self):
        if not self.main.session["black_theme"]:
            self.main.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())
            self.main.post_gui.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())
            self.main.parent.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())
            self.main.session["black_theme"] = not self.main.session["black_theme"]
        else:
            self.main.setStyleSheet("")
            self.main.post_gui.setStyleSheet("")
            self.main.parent.setStyleSheet("")
            self.main.session["black_theme"] = not self.main.session["black_theme"]

    def openPostGui(self):
        self.main.post_gui.showMaximized()

    def doFullScreen(self):
        """
        Toggles full-screen mode.

        Hides or shows specific GUI elements in full-screen mode.
        """
        if self.action_full_screen.isChecked():
            self.main.history_list.hide()
            self.main.sequence_list.hide()
            self.main.sequence_inputs.hide()
            self.main.console.hide()
            self.main.input_table.hide()
            self.main.custom_and_protocol.hide()
        else:
            self.main.history_list.show()
            self.main.sequence_list.show()
            self.main.sequence_inputs.show()
            self.main.console.show()
            self.main.input_table.show()
            self.main.custom_and_protocol.show()

    def doScreenshot(self):
        """
        Takes a screenshot of the main GUI and saves it.

        The screenshot is saved in the specified screenshot folder with a timestamp as the filename.
        """
        name = datetime.now()
        name_string = name.strftime("%Y.%m.%d.%H.%M.%S.%f")[:-3]
        file_name = name_string+".png"
        screenshot = QPixmap(self.main.size())
        self.main.render(screenshot)

        # Create screenshot folder
        screenshot_folder = self.main.session['directory'] + "/screenshots"
        if not os.path.exists(screenshot_folder):
            os.makedirs(screenshot_folder)

        # Save screenshot and print message
        screenshot.save(screenshot_folder+"/"+file_name)
        print("Screenshot saved in " + screenshot_folder+"/"+file_name)

    def open_folder(self):
        # Get the current operating system
        current_os = platform.system()

        try:
            if current_os == 'Windows':
                # Open folder on Windows
                os.startfile(os.path.normpath(self.main.session['directory']))
            elif current_os == 'Darwin':  # macOS
                # Open folder on macOS
                subprocess.run(['open', self.main.session['directory']])
            elif current_os == 'Linux':
                # Open folder on Linux
                subprocess.run(['xdg-open', self.main.session['directory']])
            else:
                print(f"Unsupported OS: {current_os}")
        except Exception as e:
            print(f"Error opening folder: {e}")


class FiguresControllerPos(FiguresToolBar):
    """
    Controller class for managing figures and screenshots.

    Args:
        *args: Variable length argument list.
        **kwargs: Arbitrary keyword arguments.

    Inherits:
        FiguresToolBar: Base class for the figures toolbar.
    """
    def __init__(self, *args, **kwargs):
        """
        Initializes the FiguresController.

        Args:
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.
        """
        super(FiguresControllerPos, self).__init__(*args, **kwargs)

        # Hide post button:
        self.action_postprocessing.setVisible(False)
        self.action_open_directory.setVisible(False)

        self.action_full_screen.setCheckable(True)
        self.action_full_screen.triggered.connect(self.doFullScreen)
        self.action_screenshot.triggered.connect(self.doScreenshot)
        self.switch_theme_action.triggered.connect(self.switch_theme)

    def switch_theme(self):
        if not self.main.session["black_theme"]:
            self.main.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())
            self.main.main.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())
            self.main.main.parent.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())
            self.main.session["black_theme"] = not self.main.session["black_theme"]
        else:
            self.main.setStyleSheet("")
            self.main.main.setStyleSheet("")
            self.main.main.parent.setStyleSheet("")
            self.main.session["black_theme"] = not self.main.session["black_theme"]

    def doFullScreen(self):
        """
        Toggles full-screen mode.

        Hides or shows specific GUI elements in full-screen mode.
        """
        if self.action_full_screen.isChecked():
            self.main.history_list.hide()
            self.main.methods_list.hide()
            self.main.tab_controller.hide()
            self.main.main.console.hide()
        else:
            self.main.history_list.show()
            self.main.methods_list.show()
            self.main.tab_controller.show()
            self.main.main.console.show()

    def doScreenshot(self):
        """
        Takes a screenshot of the main GUI and saves it.

        The screenshot is saved in the specified screenshot folder with a timestamp as the filename.
        """
        name = datetime.now()
        name_string = name.strftime("%Y.%m.%d.%H.%M.%S.%f")[:-3]
        file_name = name_string+".png"
        screenshot = QPixmap(self.main.size())
        self.main.render(screenshot)

        # Create screenshot folder
        screenshot_folder = self.main.session['directory'] + "/screenshots"
        if not os.path.exists(screenshot_folder):
            os.makedirs(screenshot_folder)

        # Save screenshot and print message
        screenshot.save(screenshot_folder+"/"+file_name)
        print("Screenshot saved in " + screenshot_folder+"/"+file_name)



#!/usr/bin/env python3
#
# Compatibility layer vaguely similar to a certain other company's
# pulse programming language
#

import numpy as np
import experiment as exp

class Marcostek:
    """Provides a simple API to interact with the Experiment class, with
    discrete commands that can be called in order."""

    def __init__(self, exp, grad_update_interval=5,
                 tx_gate_overhead=0,
                 invert_tx_gate=False,
                 rx_gate_overhead=0,
                 invert_rx_gate=False):
        """exp: previously-created Experiment object

        grad_update_interval: raster time of gradients in us.

        Must be equal to or greater than 1 / grad_max_update_rate used
        to create the exp object (by default 0.2 MSPS, so
        grad_update_interval must be > 5 for OCRA1 and > 1.25 for
        GPA-FHDO)

        tx_gate_overhead: how far in advance of an RF pulse to turn the TX gate TTL on

        invert_tx_gate: invert TX gate TTL polarity (if true, logic 0 during TX)

        rx_gate_overhead: how far in advance of an acquisition (RX) pulse to turn the RX gate TTL on

        invert_rx_gate: invert RX gate TTL polarity (if true, logic 0 during RX)
        """
        self._exp = exp
        self._grad_update_interval = grad_update_interval

        assert tx_gate_overhead >= 0, "TX gate overhead time cannot be negative"
        self._tx_gate_overhead = tx_gate_overhead
        self._invert_tx_gate = invert_tx_gate

        assert rx_gate_overhead >= 0, "RX gate overhead time cannot be negative"
        self._rx_gate_overhead = rx_gate_overhead
        self._invert_rx_gate = invert_rx_gate
        self._global_time = grad_update_interval


    def _chan_str(self, chan):
        chan_str = ['x', 'y', 'z', 'z2']
        if type(chan) is str:
            assert chan in chan_str, "Unknown grad channel"
            return chan
        else:
            assert 0 <= chan and chan <= 3, "Unknown grad channel index"
            return chan_str[chan]

    ### Pulse programming commands

    def delay(self, time):
        """ time in us """
        assert time > 0, "Time cannot be negative"
        self._global_time += time

    def gradoff(self, chan):
        """sets a gradient channel to 0V for the ocra1, 0A for the gpa-fhdo.
        chan: 0, 1, 2 or 3, or 'x', 'y', 'z' or 'z2'

        Takes grad_update_interval us to complete.
        """
        self._exp.add_flodict({'grad_v' + self._chan_str(chan):
                               ( np.array([self._global_time]), np.array([0]) )})

        self._global_time += self._grad_update_interval

    def gradon(self, chan, value):
        """sets a gradient channel to value
        chan: 0, 1, 2 or 3, or 'x', 'y', 'z' or 'z2'
        value: between -1 and +1; +1 = maximum gradient output, -1 = maximum negative gradient output

        Takes grad_update_interval us to complete.
        """
        assert -1 <= value and value <= 1, "Grad value out of range"
        self._exp.add_flodict({'grad_v' + self._chan_str(chan):
                               ( np.array([self._global_time]), np.array([value]) )})

        self._global_time += self._grad_update_interval

    def gradramp(self, chan, start_val, end_val, n_steps, step_duration):
        """Creates a ramp on a gradient channel
        chan: 0, 1, 2 or 3, or 'x', 'y', 'z' or 'z2'

        start_val and end_val: between -1 and +1; +1 = maximum
        gradient output, -1 = maximum negative gradient output

        n_steps: number of output steps

        step_duration: time duration of each step, at least grad_update_interval

        Outputs will occur at times step_duration, 2*step_duration,
        etc with values of start_val + (end_val - start_val) * k /
        (n_steps - 1), where k goes from 0 to n_steps - 1

        Note that the total ramp length is step_duration * n_steps
        """
        assert -1 <= start_val and start_val <= 1, "Grad ramp start value out of range"
        assert -1 <= end_val and end_val <= 1, "Grad ramp end value out of range"
        assert n_steps > 0, "Number of steps cannot be negative"
        assert step_duration >= self._grad_update_interval, \
            "Step duration cannot be shorter than gradient update interval"
        self._exp.add_flodict({'grad_v' + self._chan_str(chan):
                               ( self._global_time + np.linspace(step_duration, n_steps * step_duration, n_steps),
                                 np.linspace(start_val, end_val, n_steps) )})
        self._global_time += n_steps * step_duration

    def pulse(self, chan, amp, phase, duration, end_amp=0, end_phase=0, pulse_tx_gate=True, tx_gate_overhead=None):
        """Does a rectangular RF pulse

        chan: 0 or 1, marga TX channel to output the pulse on

        amp: output amplitude, proportional to voltage; 1 = full-scale

        phase: RF phase (w.r.t. carrier tone) in degrees; wrapped to [0, 360) if out of range

        duration: length of RF pulse

        end_amp: final output amplitude to return to, if a nonzero
        value is desired (can be used to create pulses that take place
        while other events are occurring)

        end_phase: phase used for end_amp in degrees; wrapped to [0, 360) if out of range

        pulse_tx_gate: pulse the TX gate TTL output during the RF pulse

        tx_gate_overhead: time before RF pulse begins to turn on the TX gate TTL

        Note: total delay of the command is duration + tx_gate_overhead

        Note: if pulse_tx_gate is true and two pulses are run one
        after the other, the TX gate value is undefined at the instant
        between pulses (one is trying to switch it off, while the
        other is trying to switch it on). To avoid this happening,
        either add a delay between pulses, or make sure that one of
        the pulses' pulse_tx_gate parameter is set to False.
        """
        if tx_gate_overhead is None:
            tx_gate_overhead = self._tx_gate_overhead
        assert chan in [0, 1], "Invalid RF channel"
        assert 0 <= amp and amp <= 1, "RF amplitude out of range"
        assert 0 <= end_amp and amp <= 1, "RF end amplitude out of range"
        assert duration > 0, "RF pulse duration cannot be negative"
        assert tx_gate_overhead >= 0, "RF pulse gate overhead time cannot be negative"

        phase_rad, end_phase_rad = phase * np.pi/180, end_phase * np.pi/180
        cplx_scale = np.cos(phase_rad) + 1j * np.sin(phase_rad)
        end_cplx_scale = np.cos(end_phase_rad) + 1j * np.sin(end_phase_rad)

        self._exp.add_flodict({'tx'+str(chan): ( self._global_time + tx_gate_overhead + np.array([0, duration]),
                                                 np.array([amp * cplx_scale, end_amp * end_cplx_scale]) )})

        if pulse_tx_gate:
            self._exp.add_flodict({'tx_gate': ( self._global_time + np.array([0, duration + tx_gate_overhead]),
                                                np.array([not self._invert_tx_gate, self._invert_tx_gate]) )})

        self._global_time += duration + tx_gate_overhead

    def rx(self, chan, duration, pulse_rx_gate=True, rx_gate_overhead=None):
        """Acquire RX data for the duration specified, at the RX rate
        specified when the Experiment object was created.

        chan: 0 or 1, marga RX channel to sample

        duration: length of acquisition

        pulse_rx_gate: pulse the RX gate TTL output during the acquisition

        rx_gate_overhead: time before acquisition begins to turn on the RX gate TTL

        Note: total delay of the command is duration + rx_gate_overhead
        """

        if rx_gate_overhead is None:
            rx_gate_overhead = self._rx_gate_overhead
        assert chan in [0, 1], "Invalid RF channel"
        assert duration > 0, "RX duration cannot be negative"
        assert rx_gate_overhead >= 0, "RX gate overhead time cannot be negative"

        self._exp.add_flodict({
            'rx'+str(chan)+'_en': (self._global_time + rx_gate_overhead + np.array([0, duration]),
                                   np.array([1, 0]) )})

        if pulse_rx_gate:
            self._exp.add_flodict({'rx_gate': (self._global_time + np.array([0, duration + rx_gate_overhead]),
                                               np.array([not self._invert_rx_gate, self._invert_rx_gate]) ) })

        self._global_time += duration + rx_gate_overhead

def test_marcostek():
    expt = exp.Experiment(lo_freq=5, # MHz
                          rx_t=1.5) # us sampling rate)
    f = Marcostek(expt)

    # Turn all 4 gradients off
    for k in range(2):
        f.gradoff(k) # index-based
    f.gradoff('z') # string-based
    f.gradoff('z2')

    # Wait 10us
    f.delay(10)

    # Turn gradients on, 5us apart, to -1, -0.5, 0.3 and 0.8x full-scale
    for k, m in enumerate([-1, -0.5, 0.3, 0.8]):
        f.gradon(k, m)

    # Ramp the x and y gradients one after the other: start value, end value, number of steps, step duration
    f.delay(20)
    f.gradramp('x', 0.0, 0.8, 5, 8)
    f.gradramp('y', 0.0, 0.6, 5, 8)

    f.delay(20)
    f.gradramp('x', 0.8, -0.5, 10, 5)
    f.gradramp('y', 0.6, -0.5, 10, 5)

    # Do some RF pulses of different amplitudes and lengths: 0, 180 and 235 deg phases
    # Channel, amplitude, phase, pulse length
    first_time = 50
    second_time = 30
    third_time = 70
    f.delay(20)
    f.pulse(0, 0.5, 0, first_time)
    f.delay(10)
    f.pulse(1, 0.3, 0, first_time)

    f.delay(10)
    f.pulse(0, 0.9, 180, second_time, pulse_tx_gate=False)
    f.delay(20)
    f.pulse(1, 0.8, 180, second_time)

    f.delay(30)
    f.pulse(0, 0.4, 235, third_time)
    f.delay(15)
    f.pulse(1, 0.6, 235, third_time)

    # Acquire data on channel 0 for 100us (at 1.5us sampling rate, specified in the construction of expt)
    f.rx(0, 100)

    # Turn off gradients
    for k in range(4):
        f.gradoff(k)

    rxd, msgs = expt.run()
    expt.close_server(True)
    expt._s.close() # close socket

if __name__ == "__main__":
    test_marcostek()



import numpy as np
from PyQt5.QtWidgets import QGridLayout
from marge.widgets.widget_visualisation import VisualisationTabWidget
from controller.controller_plot3d import Plot3DController as Spectrum3DPlot


class VisualisationTabController(VisualisationTabWidget):
    """
    Controller class for the VisualisationTabWidget.

    Inherits from VisualisationTabWidget to provide additional functionality for managing visualisation tab actions.

    Attributes:
        visualisation_button: QPushButton for showing 2D slices.
    """

    def __init__(self, *args, **kwargs):
        """
        Initialize the VisualisationTabController.

        Args:
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.
        """
        super(VisualisationTabController, self).__init__(*args, **kwargs)

        self.layout = QGridLayout()
        # Connect the visualisation_button clicked signal to the imageVisualisation method
        self.visualisation_button.clicked.connect(self.imageVisualisation)
        self.image_view = None

    def imageVisualisation(self):
        """
        Perform image visualisation on the selected slices of the main matrix.

        Display the selected slices as images using a grid layout.
        """

        image = self.main.image_view_widget.main_matrix

        slices = self.range_text_field.text().split(',')
        n0 = int(slices[0])
        n_end = int(slices[1])
        # slices_number = n_end - n0 + 1
        selected_slices = image[n0:n_end + 1]

        rows_columns = self.column_text_field.text().split(',')
        rows_number = int(rows_columns[0])
        columns_number = int(rows_columns[1])

        slice_height, slice_width = image.shape[1], image.shape[2]

        image_matrix = np.zeros((slice_height * columns_number, slice_width * rows_number), dtype=np.complex128)

        i = 0
        while i < len(selected_slices):
            row = i // columns_number
            col = i % columns_number
            row_start = row * slice_height
            row_end = row_start + slice_height
            col_start = col * slice_width
            col_end = col_start + slice_width

            image_matrix[col_start:col_end, row_start:row_end] = image[i]

            i += 1

        # Clean the image_view_widget
        self.main.image_view_widget.clearFiguresLayout()

        # Create new widget
        image = Spectrum3DPlot(main=self.main,
                               data=np.abs(np.reshape(image_matrix, (1, image_matrix.shape[0], image_matrix.shape[1]))),
                               x_label='',
                               y_label='',
                               title='')
        self.main.image_view_widget.addWidget(image)

    def clear2DImage(self):
        """
        Clear the second image view.
        """
        if self.image_view is not None:
            self.image_view.close()
            self.image_view = None



#  NanoVNASaver
#
#  A python program to view and export Touchstone data from a NanoVNA
#  Copyright (C) 2019, 2020  Rune B. Broberg
#  Copyright (C) 2020,2021 NanoVNA-Saver Authors
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
import logging

from marge.vna.Serial import Interface
from marge.vna.VNA import VNA

logger = logging.getLogger(__name__)


class AVNA(VNA):
    name = "AVNA"

    def __init__(self, iface: Interface):
        super().__init__(iface)
        self.sweep_max_freq_Hz = 40e3
        self.features.add("Customizable data points")

    def isValid(self):
        return True

    def resetSweep(self, start: int, stop: int):
        list(self.exec_command(f"sweep {start} {stop} {self.datapoints}"))
        list(self.exec_command("resume"))

    def setSweep(self, start, stop):
        list(self.exec_command(f"sweep {start} {stop} {self.datapoints}"))
